@@ -1,15 +1,20 @@
 """Base geometry class and utilities
 """
 
-from ctypes import pointer, c_size_t, c_char_p, c_void_p
 import sys
-import warnings
+from warnings import warn
+from binascii import a2b_hex
+from ctypes import pointer, c_size_t, c_char_p, c_void_p
 
 from shapely.coords import CoordinateSequence
 from shapely.ftools import wraps
 from shapely.geos import lgeos, ReadingError
+from shapely.geos import WKBWriter, WKTWriter
 from shapely.impl import DefaultImplementation, delegated
 
+if sys.version_info[0] < 3:
+    range = xrange
+
 GEOMETRY_TYPES = [
     'Point',
     'LineString',
@@ -18,14 +23,32 @@ GEOMETRY_TYPES = [
     'MultiPoint',
     'MultiLineString',
     'MultiPolygon',
-    'GeometryCollection'
-    ]
+    'GeometryCollection',
+]
+
+
+def dump_coords(geom):
+    """Dump coordinates of a geometry in the same order as data packing"""
+    if not isinstance(geom, BaseGeometry):
+        raise ValueError('Must be instance of a geometry class; found ' +
+                         geom.__class__.__name__)
+    elif geom.type in ('Point', 'LineString', 'LinearRing'):
+        return geom.coords[:]
+    elif geom.type == 'Polygon':
+        return geom.exterior.coords[:] + [i.coords[:] for i in geom.interiors]
+    elif geom.type.startswith('Multi') or geom.type == 'GeometryCollection':
+        # Recursive call
+        return [dump_coords(part) for part in geom]
+    else:
+        raise ValueError('Unhandled geometry type: ' + repr(geom.type))
+
 
 def geometry_type_name(g):
     if g is None:
         raise ValueError("Null geometry has no type")
     return GEOMETRY_TYPES[lgeos.GEOSGeomTypeId(g)]
 
+
 def geom_factory(g, parent=None):
     # Abstract geometry factory for use with topological methods below
     if not g:
@@ -48,34 +71,50 @@ def geom_factory(g, parent=None):
         ob._ndim = 2
     return ob
 
+
 def geom_from_wkt(data):
+    warn("`geom_from_wkt` is deprecated. Use `geos.wkt_reader.read(data)`.",
+         DeprecationWarning)
+    if sys.version_info[0] >= 3:
+        data = data.encode('ascii')
     geom = lgeos.GEOSGeomFromWKT(c_char_p(data))
     if not geom:
-        raise ReadingError, \
-        "Could not create geometry because of errors while reading input."
+        raise ReadingError(
+            "Could not create geometry because of errors while reading input.")
     return geom_factory(geom)
 
+
 def geom_to_wkt(ob):
+    warn("`geom_to_wkt` is deprecated. Use `geos.wkt_writer.write(ob)`.",
+         DeprecationWarning)
     if ob is None or ob._geom is None:
         raise ValueError("Null geometry supports no operations")
     return lgeos.GEOSGeomToWKT(ob._geom)
 
+
 def deserialize_wkb(data):
-    geom = lgeos.GEOSGeomFromWKB_buf(c_char_p(data), c_size_t(len(data)));
+    geom = lgeos.GEOSGeomFromWKB_buf(c_char_p(data), c_size_t(len(data)))
     if not geom:
         raise ReadingError(
             "Could not create geometry because of errors while reading input.")
     return geom
 
+
 def geom_from_wkb(data):
+    warn("`geom_from_wkb` is deprecated. Use `geos.wkb_reader.read(data)`.",
+         DeprecationWarning)
     return geom_factory(deserialize_wkb(data))
 
+
 def geom_to_wkb(ob):
+    warn("`geom_to_wkb` is deprecated. Use `geos.wkb_writer.write(ob)`.",
+         DeprecationWarning)
     if ob is None or ob._geom is None:
         raise ValueError("Null geometry supports no operations")
     size = c_size_t()
     return lgeos.GEOSGeomToWKB_buf(c_void_p(ob._geom), pointer(size))
 
+
 def exceptNull(func):
     """Decorator which helps avoid GEOS operations on null pointers."""
     @wraps(func)
@@ -85,17 +124,20 @@ def exceptNull(func):
         return func(*args, **kwargs)
     return wrapper
 
+
 class CAP_STYLE(object):
     round = 1
     flat = 2
     square = 3
 
+
 class JOIN_STYLE(object):
     round = 1
     mitre = 2
     bevel = 3
 
-EMPTY = deserialize_wkb('010700000000000000'.decode('hex'))
+EMPTY = deserialize_wkb(a2b_hex(b'010700000000000000'))
+
 
 class BaseGeometry(object):
     """
@@ -118,15 +160,15 @@ class BaseGeometry(object):
     # _crs : object
     #     Coordinate reference system. Available for Shapely extensions, but
     #     not implemented here.
-    # _owned : bool
-    #     True if this object's GEOS geometry is owned by another as in the case
-    #     of a multipart geometry member.
+    # _other_owned : bool
+    #     True if this object's GEOS geometry is owned by another as in the
+    #     case of a multipart geometry member.
     __geom__ = EMPTY
     __p__ = None
     _ctypes_data = None
     _ndim = None
     _crs = None
-    _owned = False
+    _other_owned = False
 
     # Backend config
     impl = DefaultImplementation
@@ -138,27 +180,26 @@ class BaseGeometry(object):
     # a reference to the so/dll proxy to preserve access during clean up
     _lgeos = lgeos
 
-    def empty(self):
+    def empty(self, val=EMPTY):
         # TODO: defer cleanup to the implementation. We shouldn't be
         # explicitly calling a lgeos method here.
-        if not (self._owned or self._is_empty):
+        if not self._is_empty and not self._other_owned and self.__geom__:
             try:
                 self._lgeos.GEOSGeom_destroy(self.__geom__)
             except AttributeError:
-                pass # _lgeos might be empty on shutdown
-        self.__geom__ = EMPTY
+                pass  # _lgeos might be empty on shutdown
+        self.__geom__ = val
 
     def __del__(self):
-        self.empty()
-        self.__geom__ = None
+        self.empty(val=None)
         self.__p__ = None
 
     def __str__(self):
-        return self.to_wkt()
+        return self.wkt
 
     # To support pickling
     def __reduce__(self):
-        return (self.__class__, (), self.to_wkb())
+        return (self.__class__, (), self.wkb)
 
     def __setstate__(self, state):
         self.empty()
@@ -168,13 +209,14 @@ class BaseGeometry(object):
         else:
             self._ndim = 2
 
-    # The _geom property
-    def _get_geom(self):
+    @property
+    def _geom(self):
         return self.__geom__
-    def _set_geom(self, val):
+
+    @_geom.setter
+    def _geom(self, val):
         self.empty()
         self.__geom__ = val
-    _geom = property(_get_geom, _set_geom)
 
     # Operators
     # ---------
@@ -207,7 +249,7 @@ class BaseGeometry(object):
             typestr = '>f8'
         else:
             raise ValueError(
-                  "Unsupported byteorder: neither little nor big-endian")
+                "Unsupported byteorder: neither little nor big-endian")
         return {
             'version': 3,
             'typestr': typestr,
@@ -257,18 +299,64 @@ class BaseGeometry(object):
         return self.geometryType()
 
     def to_wkb(self):
+        warn("`to_wkb` is deprecated. Use the `wkb` property.",
+             DeprecationWarning)
         return geom_to_wkb(self)
 
     def to_wkt(self):
+        warn("`to_wkt` is deprecated. Use the `wkt` property.",
+             DeprecationWarning)
         return geom_to_wkt(self)
 
-    geom_type = property(geometryType,
-        doc="""Name of the geometry's type, such as 'Point'"""
-        )
-    wkt = property(to_wkt,
-        doc="""WKT representation of the geometry""")
-    wkb = property(to_wkb,
-        doc="""WKB representation of the geometry""")
+    @property
+    def wkt(self, **kw):
+        """WKT representation of the geometry"""
+        return WKTWriter(lgeos, **kw).write(self)
+
+    @property
+    def wkb(self):
+        """WKB representation of the geometry"""
+        return WKBWriter(lgeos).write(self)
+
+    @property
+    def wkb_hex(self):
+        """WKB hex representation of the geometry"""
+        return WKBWriter(lgeos).write_hex(self)
+
+    def svg(self, scale_factor=1.):
+        """
+        SVG representation of the geometry. Scale factor is multiplied by
+        the size of the SVG symbol so it can be scaled consistently for a
+        consistent appearance based on the canvas size.
+        """
+        raise NotImplementedError
+
+    def _repr_svg_(self):
+        """SVG representation for iPython notebook"""
+        #Pick an arbitrary size for the SVG canvas
+
+
+        xmin, ymin, xmax, ymax = self.buffer(1).bounds
+        x_size = min([max([100., xmax - xmin]), 300])
+        y_size = min([max([100., ymax - ymin]), 300])
+        try:
+            scale_factor = max([xmax - xmin, ymax - ymin]) / max([x_size, y_size])
+        except ZeroDivisionError:
+            scale_factor = 1
+        buffered_box = "{0} {1} {2} {3}".format(xmin, ymin, xmax - xmin, ymax - ymin)
+        return """<svg
+            preserveAspectRatio="xMinYMin meet"
+            viewBox="{0}"
+            width="{1}"
+            height="{2}"
+            transform="translate(0, {1}),scale(1, -1)">
+            {3}
+            </svg>""".format(buffered_box, x_size, y_size, self.svg(scale_factor))
+
+    @property
+    def geom_type(self):
+        """Name of the geometry's type, such as 'Point'"""
+        return self.geometryType()
 
     # Real-valued properties and methods
     # ----------------------------------
@@ -335,7 +423,7 @@ class BaseGeometry(object):
 
     def buffer(self, distance, resolution=16, quadsegs=None,
                cap_style=CAP_STYLE.round, join_style=JOIN_STYLE.round,
-               mitre_limit=0):
+               mitre_limit=5.0):
         """Returns a geometry with an envelope at a distance from the object's
         envelope
 
@@ -374,15 +462,16 @@ class BaseGeometry(object):
           >>> g.buffer(1.0, cap_style='square').area
           4.0
         """
-
         if quadsegs is not None:
-            warnings.warn(
+            warn(
                 "The `quadsegs` argument is deprecated. Use `resolution`.",
                 DeprecationWarning)
             res = quadsegs
         else:
             res = resolution
-
+        if mitre_limit == 0.0:
+            raise ValueError(
+                'Cannot compute offset from zero-length line segment')
         if cap_style == CAP_STYLE.round and join_style == JOIN_STYLE.round:
             return geom_factory(self.impl['buffer'](self, distance, res))
 
@@ -553,26 +642,26 @@ class BaseMultipartGeometry(BaseGeometry):
     @property
     def ctypes(self):
         raise NotImplementedError(
-        "Multi-part geometries have no ctypes representations")
+            "Multi-part geometries have no ctypes representations")
 
     @property
     def __array_interface__(self):
         """Provide the Numpy array protocol."""
-        raise NotImplementedError(
-        "Multi-part geometries do not themselves provide the array interface")
+        raise NotImplementedError("Multi-part geometries do not themselves "
+                                  "provide the array interface")
 
     def _get_coords(self):
-        raise NotImplementedError(
-        "Sub-geometries may have coordinate sequences, but collections do not")
+        raise NotImplementedError("Sub-geometries may have coordinate "
+                                  "sequences, but collections do not")
 
     def _set_coords(self, ob):
-        raise NotImplementedError(
-        "Sub-geometries may have coordinate sequences, but collections do not")
+        raise NotImplementedError("Sub-geometries may have coordinate "
+                                  "sequences, but collections do not")
 
     @property
     def coords(self):
         raise NotImplementedError(
-        "Multi-part geometries do not provide a coordinate sequence")
+            "Multi-part geometries do not provide a coordinate sequence")
 
     @property
     def geoms(self):
@@ -598,6 +687,14 @@ class BaseMultipartGeometry(BaseGeometry):
         else:
             return ()[index]
 
+    def svg(self, scale_factor=1.):
+        """
+        SVG representation of the geometry. Scale factor is multiplied by
+        the size of the SVG symbol so it can be scaled consistently for a
+        consistent appearance based on the canvas size.
+        """
+        return "\n".join([g.svg(scale_factor) for g in self])
+
 
 class GeometrySequence(object):
     """
@@ -629,7 +726,7 @@ class GeometrySequence(object):
 
     def _get_geom_item(self, i):
         g = self.shape_factory()
-        g._owned = True
+        g._other_owned = True
         g._geom = lgeos.GEOSGetGeometryN(self._geom, i)
         g._ndim = self._ndim
         g.__p__ = self
@@ -637,7 +734,7 @@ class GeometrySequence(object):
 
     def __iter__(self):
         self._update()
-        for i in xrange(self.__len__()):
+        for i in range(self.__len__()):
             yield self._get_geom_item(i)
 
     def __len__(self):
@@ -661,7 +758,7 @@ class GeometrySequence(object):
                     "Heterogenous geometry collections are not sliceable")
             res = []
             start, stop, stride = key.indices(m)
-            for i in xrange(start, stop, stride):
+            for i in range(start, stop, stride):
                 res.append(self._get_geom_item(i))
             return type(self.__p__)(res or None)
         else:
@@ -687,11 +784,12 @@ class HeterogeneousGeometrySequence(GeometrySequence):
     def _get_geom_item(self, i):
         sub = lgeos.GEOSGetGeometryN(self._geom, i)
         g = geom_factory(sub, parent=self)
-        g._owned = True
+        g._other_owned = True
         return g
 
-# Test runner
+
 def _test():
+    """Test runner"""
     import doctest
     doctest.testmod()
 
