@@ -1,6 +1,11 @@
 """Collections of linestrings and related utilities
 """
 
+import sys
+
+if sys.version_info[0] < 3:
+    range = xrange
+
 from ctypes import c_double, c_void_p, cast, POINTER
 
 from shapely.geos import lgeos
@@ -40,8 +45,8 @@ class MultiLineString(BaseMultipartGeometry):
         """
         super(MultiLineString, self).__init__()
 
-        if lines is None:
-            # allow creation of null lines, to support unpickling
+        if not lines:
+            # allow creation of empty multilinestrings, to support unpickling
             pass
         else:
             self._geom, self._ndim = geos_multilinestring_from_py(lines)
@@ -56,11 +61,32 @@ class MultiLineString(BaseMultipartGeometry):
             'coordinates': tuple(tuple(c for c in g.coords) for g in self.geoms)
             }
 
+    def svg(self, scale_factor=1.):
+        """
+        SVG representation of the geometry. Scale factor is multiplied by
+        the size of the SVG symbol so it can be scaled consistently for a
+        consistent appearance based on the canvas size.
+        """
+        parts = []
+        for part in self.geoms:
+            pnt_format = " ".join(["{0},{1}".format(*c) for c in part.coords])
+            parts.append("""<polyline
+                fill="none"
+                stroke="{2}"
+                stroke-width={1}
+                points="{0}"
+                opacity=".8"
+                />""".format(
+                    pnt_format,
+                    2.*scale_factor,
+                    "#66cc99" if self.is_valid else "#ff3333"))
+        return "\n".join(parts)
+
 
 class MultiLineStringAdapter(CachingGeometryProxy, MultiLineString):
     
     context = None
-    _owned = False
+    _other_owned = False
 
     def __init__(self, context):
         self.context = context
@@ -93,13 +119,10 @@ def geos_multilinestring_from_py(ob):
         L = array['shape'][0]
         assert L >= 1
 
-        # Make pointer to the coordinate array
-        cp = cast(array['data'][0], POINTER(c_double))
-
         # Array of pointers to sub-geometries
         subs = (c_void_p * L)()
 
-        for l in xrange(L):
+        for l in range(L):
             geom, ndims = geos_linestring_from_py(array['data'][l])
             subs[i] = cast(geom, c_void_p)
         N = lgeos.GEOSGeom_getDimensions(subs[0])
@@ -118,7 +141,7 @@ def geos_multilinestring_from_py(ob):
         subs = (c_void_p * L)()
         
         # add to coordinate sequence
-        for l in xrange(L):
+        for l in range(L):
             geom, ndims = geos_linestring_from_py(obs[l])
             subs[l] = cast(geom, c_void_p)
             
