@@ -1,93 +1,95 @@
-"""
-Multiple points.
+"""Collections of points and related utilities
 """
 
-from ctypes import byref, c_double, c_int, c_void_p, cast, POINTER, pointer
+import sys
 
+if sys.version_info[0] < 3:
+    range = xrange
+
+from ctypes import byref, c_double, c_void_p, cast, POINTER
+from ctypes import ArgumentError
+
+from shapely.coords import required
 from shapely.geos import lgeos
-from shapely.geometry.base import BaseGeometry, GeometrySequence, exceptNull
+from shapely.geometry.base import BaseMultipartGeometry, exceptNull
 from shapely.geometry.point import Point, geos_point_from_py
+from shapely.geometry.proxy import CachingGeometryProxy
+
+__all__ = ['MultiPoint', 'asMultiPoint']
 
 
-def geos_multipoint_from_py(ob):
-    try:
-        # From array protocol
-        array = ob.__array_interface__
-        assert len(array['shape']) == 2
-        m = array['shape'][0]
-        n = array['shape'][1]
-        assert m >= 1
-        assert n == 2 or n == 3
+class MultiPoint(BaseMultipartGeometry):
 
-        # Make pointer to the coordinate array
-        cp = cast(array['data'][0], POINTER(c_double))
+    """A collection of one or more points
 
-        # Array of pointers to sub-geometries
-        subs = (c_void_p * m)()
+    A MultiPoint has zero area and zero length.
 
-        for i in xrange(m):
-            geom, ndims = geos_point_from_py(cp[n*i:n*i+2])
-            subs[i] = cast(geom, c_void_p)
-
-    except AttributeError:
-        # Fall back on list
-        m = len(ob)
-        n = len(ob[0])
-        assert n == 2 or n == 3
-
-        # Array of pointers to point geometries
-        subs = (c_void_p * m)()
-        
-        # add to coordinate sequence
-        for i in xrange(m):
-            coords = ob[i]
-            geom, ndims = geos_point_from_py(coords)
-            subs[i] = cast(geom, c_void_p)
-            
-    return lgeos.GEOSGeom_createCollection(4, subs, m), n
-
-
-class MultiPoint(BaseGeometry):
-
-    """A multiple point geometry.
+    Attributes
+    ----------
+    geoms : sequence
+        A sequence of Points
     """
 
-    def __init__(self, coordinates=None):
-        """Initialize.
-
+    def __init__(self, points=None):
+        """
         Parameters
         ----------
-        
-        coordinates : sequence or array
-            This may be an object that satisfies the numpy array protocol,
-            providing an M x 2 or M x 3 (with z) array, or it may be a sequence
-            of x, y (,z) coordinate sequences.
+        points : sequence
+            A sequence of (x, y [,z]) numeric coordinate pairs or triples or a
+            sequence of objects that implement the numpy array interface,
+            including instaces of Point.
 
         Example
         -------
+        Construct a 2 point collection
 
-        >>> geom = MultiPoint([[0.0, 0.0], [1.0, 2.0]])
-        >>> geom = MultiPoint(array([[0.0, 0.0], [1.0, 2.0]]))
-        
-        Each result in a line string from (0.0, 0.0) to (1.0, 2.0).
+          >>> ob = MultiPoint([[0.0, 0.0], [1.0, 2.0]])
+          >>> len(ob.geoms)
+          2
+          >>> type(ob.geoms[0]) == Point
+          True
         """
-        BaseGeometry.__init__(self)
+        super(MultiPoint, self).__init__()
 
-        if coordinates is None:
-            # allow creation of null lines, to support unpickling
+        if points is None:
+            # allow creation of empty multipoints, to support unpickling
             pass
         else:
-            self._geom, self._ndim = geos_multipoint_from_py(coordinates)
+            self._geom, self._ndim = geos_multipoint_from_py(points)
 
+    def shape_factory(self, *args):
+        return Point(*args)
 
     @property
-    @exceptNull
     def __geo_interface__(self):
         return {
             'type': 'MultiPoint',
             'coordinates': tuple([g.coords[0] for g in self.geoms])
             }
 
+    def svg(self, scale_factor=1.):
+        """
+        SVG representation of the geometry. Scale factor is multiplied by
+        the size of the SVG symbol so it can be scaled consistently for a
+        consistent appearance based on the canvas size.
+        """
+        
+        parts = []
+        for part in self.geoms:
+            parts.append("""<circle
+            cx="{0.x}"
+            cy="{0.y}"
+            r="{1}"
+            stroke="#555555"
+            stroke-width="{2}"
+            fill="{3}"
+            opacity=".6"
+            />""".format(
+                part,
+                3*scale_factor,
+                1*scale_factor, "#66cc99" if self.is_valid else "#ff3333"))
+        return "\n".join(parts)
+
     @property
     @exceptNull
     def ctypes(self):
@@ -97,7 +99,7 @@ class MultiPoint(BaseGeometry):
             m = len(self.geoms)
             array_type = c_double * (m * n)
             data = array_type()
-            for i in xrange(m):
+            for i in range(m):
                 g = self.geoms[i]._geom    
                 cs = lgeos.GEOSGeom_getCoordSeq(g)
                 lgeos.GEOSCoordSeq_getX(cs, 0, byref(temp))
@@ -118,37 +120,15 @@ class MultiPoint(BaseGeometry):
         return ai
     __array_interface__ = property(array_interface)
 
-    def _get_coords(self):
-        raise NotImplementedError, \
-        "Component rings have coordinate sequences, but the polygon does not"
 
-    def _set_coords(self, ob):
-        raise NotImplementedError, \
-        "Component rings have coordinate sequences, but the polygon does not"
+class MultiPointAdapter(CachingGeometryProxy, MultiPoint):
 
-    @property
-    def coords(self):
-        raise NotImplementedError, \
-        "Multipart geometries do not themselves provide coordinate sequences"
-
-    @property
-    @exceptNull
-    def geoms(self):
-        return GeometrySequence(self, Point)
-        
-
-class MultiPointAdapter(MultiPoint):
-
-    """Adapts a Python coordinate pair or a numpy array to the multipoint
-    interface.
-    """
-    
     context = None
-    #__geom__ = None
-    _owned = False
+    _other_owned = False
 
     def __init__(self, context):
         self.context = context
+        self.factory = geos_multipoint_from_py
 
     @property
     def _ndim(self):
@@ -162,14 +142,6 @@ class MultiPointAdapter(MultiPoint):
             # Fall back on list
             return len(self.context[0])
 
-    @property
-    def _geom(self):
-        """Keeps the GEOS geometry in synch with the context."""
-        if self.__geom__ is not None:
-            lgeos.GEOSGeom_destroy(self.__geom__)
-        self.__geom__, n = geos_multipoint_from_py(self.context)
-        return self.__geom__
-
     @property
     def __array_interface__(self):
         """Provide the Numpy array protocol."""
@@ -180,10 +152,57 @@ class MultiPointAdapter(MultiPoint):
 
 
 def asMultiPoint(context):
-    """Factory for MultiPointAdapter instances."""
+    """Adapt a sequence of objects to the MultiPoint interface"""
     return MultiPointAdapter(context)
 
 
+def geos_multipoint_from_py(ob):
+    # If numpy is present, we use numpy.require to ensure that we have a
+    # C-continguous array that owns its data. View data will be copied.
+    ob = required(ob)
+    try:
+        # From array protocol
+        array = ob.__array_interface__
+        assert len(array['shape']) == 2
+        m = array['shape'][0]
+        n = array['shape'][1]
+        assert m >= 1
+        assert n == 2 or n == 3
+
+        # Make pointer to the coordinate array
+        if isinstance(array['data'], tuple):
+            # numpy tuple (addr, read-only)
+            cp = cast(array['data'][0], POINTER(c_double))
+        else:
+            cp = array['data']
+
+        # Array of pointers to sub-geometries
+        subs = (c_void_p * m)()
+
+        for i in range(m):
+            geom, ndims = geos_point_from_py(cp[n*i:n*i+2])
+            subs[i] = cast(geom, c_void_p)
+
+    except AttributeError:
+        # Fall back on list
+        m = len(ob)
+        try:
+            n = len(ob[0])
+        except TypeError:
+            n = ob[0]._ndim
+        assert n == 2 or n == 3
+
+        # Array of pointers to point geometries
+        subs = (c_void_p * m)()
+        
+        # add to coordinate sequence
+        for i in range(m):
+            coords = ob[i]
+            geom, ndims = geos_point_from_py(coords)
+            subs[i] = cast(geom, c_void_p)
+            
+    return lgeos.GEOSGeom_createCollection(4, subs, m), n
+
 # Test runner
 def _test():
     import doctest
@@ -192,4 +211,3 @@ def _test():
 
 if __name__ == "__main__":
     _test()
-
