@@ -1,6 +1,11 @@
 """Collections of polygons and related utilities
 """
 
+import sys
+
+if sys.version_info[0] < 3:
+    range = xrange
+
 from ctypes import c_void_p, cast
 
 from shapely.geos import lgeos
@@ -75,11 +80,35 @@ class MultiPolygon(BaseMultipartGeometry):
             'coordinates': allcoords
             }
 
+    def svg(self, scale_factor=1.):
+        """
+        SVG representation of the geometry. Scale factor is multiplied by
+        the size of the SVG symbol so it can be scaled consistently for a
+        consistent appearance based on the canvas size.
+        """
+        parts = []
+        for part in self.geoms:
+            exterior_coords = [["{0},{1}".format(*c) for c in part.exterior.coords]]
+            interior_coords = [
+                ["{0},{1}".format(*c) for c in interior.coords]
+                for interior in part.interiors ]
+            path = " ".join([
+                "M {0} L {1} z".format(coords[0], " L ".join(coords[1:]))
+                for coords in exterior_coords + interior_coords ])
+            parts.append(
+                """<g fill-rule="evenodd" fill="{2}" stroke="#555555"
+                stroke-width="{0}" opacity="0.6">
+                <path d="{1}" /></g>""".format(
+                    2. * scale_factor,
+                    path,
+                    "#66cc99" if self.is_valid else "#ff3333"))
+        return "\n".join(parts)
+
 
 class MultiPolygonAdapter(CachingGeometryProxy, MultiPolygon):
     
     context = None
-    _owned = False
+    _other_owned = False
 
     def __init__(self, context, context_type='polygons'):
         self.context = context
@@ -115,7 +144,7 @@ def geos_multipolygon_from_py(ob):
     assert N == 2 or N == 3
 
     subs = (c_void_p * L)()
-    for l in xrange(L):
+    for l in range(L):
         geom, ndims = geos_polygon_from_py(ob[l][0], ob[l][1:])
         subs[l] = cast(geom, c_void_p)
             
@@ -136,7 +165,7 @@ def geos_multipolygon_from_polygons(ob):
     assert N == 2 or N == 3
 
     subs = (c_void_p * L)()
-    for l in xrange(L):
+    for l in range(L):
         shell = getattr(obs[l], 'exterior', None)
         if shell is None:
             shell = obs[l][0]
