@@ -74,6 +74,26 @@ class Point(BaseGeometry):
             'coordinates': self.coords[0]
             }
 
+    def svg(self, scale_factor=1.):
+        """
+        SVG representation of the geometry. Scale factor is multiplied by
+        the size of the SVG symbol so it can be scaled consistently for a
+        consistent appearance based on the canvas size.
+        """
+        return """<circle
+            cx="{0.x}"
+            cy="{0.y}"
+            r="{1}"
+            stroke="#555555"
+            stroke-width="{2}"
+            fill="{3}"
+            opacity=".6"
+            />""".format(
+                self,
+                3 * scale_factor,
+                1 * scale_factor,
+                "#66cc99" if self.is_valid else "#ff3333")
+
     @property
     def ctypes(self):
         if not self._ctypes_data:
@@ -126,7 +146,7 @@ class Point(BaseGeometry):
 
 class PointAdapter(CachingGeometryProxy, Point):
 
-    _owned = False
+    _other_owned = False
 
     def __init__(self, context):
         self.context = context
