@@ -1,6 +1,11 @@
 """Polygons and their linear ring components
 """
 
+import sys
+
+if sys.version_info[0] < 3:
+    range = xrange
+
 from ctypes import c_double, c_void_p, cast, POINTER
 from ctypes import ArgumentError
 import weakref
@@ -120,7 +125,7 @@ class InteriorRingSequence(object):
         self._length = self.__len__()
         return self
 
-    def next(self):
+    def __next__(self):
         if self._index < self._length:
             ring = self._get_ring(self._index)
             self._index += 1
@@ -128,6 +133,9 @@ class InteriorRingSequence(object):
         else:
             raise StopIteration 
 
+    if sys.version_info[0] < 3:
+        next = __next__
+
     def __len__(self):
         return lgeos.GEOSGetNumInteriorRings(self._geom)
 
@@ -144,7 +152,7 @@ class InteriorRingSequence(object):
         elif isinstance(key, slice):
             res = []
             start, stop, stride = key.indices(m)
-            for i in xrange(start, stop, stride):
+            for i in range(start, stop, stride):
                 res.append(self._get_ring(i))
             return res
         else:
@@ -170,7 +178,7 @@ class InteriorRingSequence(object):
             ring = LinearRing()
             ring.__geom__ = g
             ring.__p__ = self
-            ring._owned = True
+            ring._other_owned = True
             ring._ndim = self._ndim
             self.__rings__[i] = weakref.ref(ring)
         return self.__rings__[i]()
@@ -229,7 +237,7 @@ class Polygon(BaseGeometry):
             ring = LinearRing()
             ring.__geom__ = g
             ring.__p__ = self
-            ring._owned = True
+            ring._other_owned = True
             ring._ndim = self._ndim
             self._exterior = weakref.ref(ring)
         return self._exterior()
@@ -274,6 +282,26 @@ class Polygon(BaseGeometry):
             'coordinates': tuple(coords)
             }
 
+    def svg(self, scale_factor=1.):
+        """
+        SVG representation of the geometry. Scale factor is multiplied by
+        the size of the SVG symbol so it can be scaled consistently for a
+        consistent appearance based on the canvas size.
+        """
+        exterior_coords = [["{0},{1}".format(*c) for c in self.exterior.coords]]
+        interior_coords = [
+            ["{0},{1}".format(*c) for c in interior.coords]
+            for interior in self.interiors ]
+        path = " ".join([
+            "M {0} L {1} z".format(coords[0], " L ".join(coords[1:]))
+            for coords in exterior_coords + interior_coords ])
+        return """
+            <g fill-rule="evenodd" fill="{2}" stroke="#555555" 
+            stroke-width="{0}" opacity="0.6">
+            <path d="{1}" />
+            </g>""".format(
+                2.*scale_factor, path, "#66cc99" if self.is_valid else "#ff3333")
+
 
 class PolygonAdapter(PolygonProxy, Polygon):
     
@@ -354,7 +382,7 @@ def geos_linearring_from_py(ob, update_geom=None, update_ndim=0):
             cs = lgeos.GEOSCoordSeq_create(M, n)
 
         # add to coordinate sequence
-        for i in xrange(m):
+        for i in range(m):
             # Because of a bug in the GEOS C API, 
             # always set X before Y
             lgeos.GEOSCoordSeq_setX(cs, i, cp[n*i])
@@ -373,7 +401,12 @@ def geos_linearring_from_py(ob, update_geom=None, update_ndim=0):
             
     except AttributeError:
         # Fall back on list
-        m = len(ob)
+        try:
+            m = len(ob)
+        except TypeError:  # Iterators, e.g. Python 3 zip
+            ob = list(ob)
+            m = len(ob)
+
         n = len(ob[0])
         if m < 3:
             raise ValueError(
@@ -397,7 +430,7 @@ def geos_linearring_from_py(ob, update_geom=None, update_ndim=0):
             cs = lgeos.GEOSCoordSeq_create(M, n)
         
         # add to coordinate sequence
-        for i in xrange(m):
+        for i in range(m):
             coords = ob[i]
             # Because of a bug in the GEOS C API, 
             # always set X before Y
@@ -447,7 +480,7 @@ def geos_polygon_from_py(shell, holes=None):
             geos_holes = (c_void_p * L)()
     
             # add to coordinate sequence
-            for l in xrange(L):
+            for l in range(L):
                 geom, ndim = geos_linearring_from_py(ob[l])
                 geos_holes[l] = cast(geom, c_void_p)
         else:
